<?php

namespace Drupal\mattermost_logger;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use GuzzleHttp\Client;
use Drupal\Core\Logger\LoggerChannelTrait;

/**
 * Mattermost logger.
 */
class MattermostLogger {
  use LoggerChannelTrait;
  use StringTranslationTrait;

  public const ERROR = 'error';
  public const WARNING = 'warning';
  public const INFO = 'info';

  /**
   * The webhook URL.
   *
   * @var ?string
   */
  protected ?string $webhookUrl;

  public function __construct(protected Client $httpClient, protected ConfigFactoryInterface $configFactory) {
    $this->webhookUrl = $this->configFactory->get('mattermost_logger.settings')->get('webhook_url');
  }

  /**
   * Send a message to Mattermost.
   *
   * @param string $channel
   *   The name of the module that is sending the message.
   * @param string $message
   *   The message to send.
   * @param string $message_type
   *   The type of message to send. (default: INFO)
   */
  public function message(string $channel, string $message, string $message_type = self::INFO): void {
    if (!$this->webhookUrl) {
      $this->getLogger($channel)->error($this->t('Mattermost webhook URL is not set.'));
      return;
    }

    $mattermost_emoji = match ($message_type) {
      self::ERROR => ':x:',
      self::WARNING => ':warning:',
      default => ':information_source:',
    };

    $message_type = strtoupper($message_type);

    $payload = [
      'text' => "$mattermost_emoji [$channel] $message_type: $message",
    ];

    try {
      $this->httpClient->post($this->webhookUrl, [
        'json' => $payload,
      ]);
    }
    catch (\Exception $e) {
      $this->getLogger($channel)->error($this->t('Failed to send Mattermost notification: @message', ['@message' => $e->getMessage()]));
    }
  }

  /**
   * Sends an error message to Mattermost.
   *
   * @param string $channel
   *   The name of the module that is sending the message.
   * @param string $message
   *   The message to send.
   */
  public function error(string $channel, string $message): void {
    $this->message($channel, $message, self::ERROR);
  }

  /**
   * Sends a warning message to Mattermost.
   *
   * @param string $channel
   *   The name of the module that is sending the message.
   * @param string $message
   *   The message to send.
   */
  public function warning(string $channel, string $message): void {
    $this->message($channel, $message, self::WARNING);
  }

}
